@@ -34,7 +34,7 @@ r'''############################################################################
 
 ###################################################################################
 
-__version__ = "25.7.16"
+__version__ = "25.7.18"
 
 print('=' * 70)
 print('MIDI Doctor')
@@ -135,6 +135,25 @@ text_events = ['text_event',
 
 ###################################################################################
 
+_TEXT_ENCODINGS = [
+    'iso-8859-1',   # Western Europe
+    'windows-1251', # Cyrillic (Russian)
+    'koi8-r',       # Cyrillic (Russian)
+    'iso-8859-5',   # Cyrillic
+    'cp866',        # DOS Cyrillic
+
+    'shift_jis',    # Japanese
+    'euc_jp',       # Japanese
+    'gb18030',      # Simplified Chinese
+    'big5',         # Traditional Chinese
+    'euc_kr',       # Korean
+
+    'utf-8',        # Universal fallback
+    'utf-16',       # Wide-char fallback
+]
+
+###################################################################################
+
 def read_midi(midi_file,
               ignore_bad_signature=True,
               ignore_bad_header=False
@@ -592,32 +611,70 @@ def repair_flat_dynamics(notes,
     
 ###################################################################################
 
-def convert_bytes_in_nested_list(lst, 
-                                 encoding='utf-8', 
-                                 errors='ignore',
-                                 return_changed_events_count=False
+def convert_bytes_in_nested_list(lst,
+                                encodingNone,
+                                errors='ignore',
+                                return_changed_events_count=False
                                 ):
-    
-    new_list = []
 
+    new_list = []
     ce_count = 0
-    
+
     for item in lst:
         if isinstance(item, list):
-            new_list.append(convert_bytes_in_nested_list(item))
+
+            result = convert_bytes_in_nested_list(item, 
+                                                  encoding, 
+                                                  errors, True
+                                                 )
             
+            sublist, subcount = result
+            new_list.append(sublist)
+            ce_count += subcount
+
         elif isinstance(item, bytes):
-            new_list.append(item.decode(encoding, errors=errors))
-            ce_count += 1
             
+            b = item
+            chosen_str = None
+            fallback_str = None
+
+            for enc in _TEXT_ENCODINGS:
+                try:
+                    s = b.decode(enc)
+                    
+                except (UnicodeDecodeError, LookupError):
+                    continue
+
+                if s.encode(enc) == b:
+                    chosen_str = s
+                    break
+
+                if fallback_str is None:
+                    fallback_str = s
+
+            if chosen_str is not None:
+                decoded = chosen_str
+                
+            elif fallback_str is not None:
+                decoded = fallback_str
+                
+            else:
+                if encoding is None:
+                    decoded = b.decode('utf-8', errors)
+
+                else:
+                    decoded = b.decode(encoding, errors)
+
+            new_list.append(decoded)
+            ce_count += 1
+
         else:
             new_list.append(item)
-            
-    if return_changed_events_count:       
-        return new_list, ce_count
 
-    else:
-        return new_list
+    if return_changed_events_count:
+        return new_list, ce_count
+        
+    return new_list
 
 ###################################################################################
 
